# coding=utf-8
from django.conf import settings
from django.shortcuts import redirect, render
from django.views.generic.base import ContextMixin

from web.models.constants import GLOBAL_STUDY_ID
from .notifications import get_notifications
from web.models import Worker, Study
from web.decorators import PermissionDecorator

handler404 = 'web.views.e404_page_not_found'
handler500 = 'web.views.e500_error'
handler403 = 'web.views.e403_permission_denied'
handler400 = 'web.views.e400_bad_request'


def index(request):
    if request.user.is_authenticated:
        return redirect('web.views.appointments')
    return redirect(getattr(settings, "LOGIN_URL"))


def e404_page_not_found(request, context=None):
    return render(request, "errors/404.html", context, status=404)


def e500_error(request, context=None):
    return render(request, "errors/500.html", context, status=500)


def e403_permission_denied(request, context=None):
    return render(request, "errors/403.html", context, status=403)


def e400_bad_request(request, context=None):
    return render(request, "errors/400.html", context, status=400)


def wrap_response(request, template, params):
    final_params = extend_context(params, request)
    return render(request, template, final_params)


def extend_context(params, request):
    study = Study.get_by_id(GLOBAL_STUDY_ID)
    person = Worker.get_by_user(request.user) # None if AnonymousUser or no Worker associated
    permissions = set()
    if person is not None:
        role = person.role
        permissions = person.get_permissions(study)
        person = str(person)
    else:
        #use full name if available, username otherwise
        if len(request.user.get_full_name()) > 1:
            person = request.user.get_full_name()
        else:
            person = request.user.get_username()
        role   = '<No worker information>'
    notifications = get_notifications(request.user)
    final_params = params.copy()
    final_params.update({
        'permissions' : permissions,
        'conf_perms'  : permissions & PermissionDecorator.codename_groups['configuration'],
        'equipment_perms'  : permissions & PermissionDecorator.codename_groups['equipment'],
        'person': person,
        'role': role,
        'notifications': notifications,
        'study_id': GLOBAL_STUDY_ID,
        'study' : study
    })
    return final_params


class WrappedView(ContextMixin):
    def get_context_data(self, **kwargs):
        context = super(WrappedView, self).get_context_data(**kwargs)
        return extend_context(context, self.request)

    def dispatch(self, *args, **kwargs):
        return super(WrappedView, self).dispatch(*args, **kwargs)


from . import auth
from . import appointment
from . import visit
from . import worker
from . import subject
from . import equipment
from . import equipment_and_rooms
from . import flying_teams
from . import kit
from . import mails
from . import statistics
from . import export
from . import contact_attempt
from . import configuration_item
from . import language
from . import voucher
from . import voucher_partner_session
from . import voucher_type
from . import voucher_type_price
from . import redcap
from . import rooms
from . import uploaded_files
from . import study
from . import password
from . import appointment_type
from . import provenance